@@ -2,8 +2,8 @@ package com.github.mustachejava;
 
 import com.github.mustachejava.util.Node;
 
-import java.io.IOException;
 import java.io.Writer;
+import java.util.List;
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicInteger;
 
@@ -11,9 +11,7 @@ import java.util.concurrent.atomic.AtomicInteger;
  * Code objects that are executed in order to evaluate the template
  */
 public interface Code {
-  Writer execute(Writer writer, Object scope);
-
-  Writer execute(Writer writer, Object[] scopes);
+  Writer execute(Writer writer, List<Object> scopes);
 
   void identity(Writer writer);
 
@@ -25,18 +23,12 @@ public interface Code {
 
   void init();
 
+  @SuppressWarnings("CloneDoesntDeclareCloneNotSupportedException")
   Object clone();
 
   Object clone(Set<Code> seen);
 
   String getName();
 
-  /**
-   * If it returns a node, that means that it successfully parsed it
-   * and advanced the reader.
-   *
-   * @param reader
-   * @return
-   */
   Node invert(Node node, String text, AtomicInteger position);
 }
