@@ -5,6 +5,7 @@ import com.github.mustachejava.Mustache;
 import com.github.mustachejava.TemplateContext;
 
 import java.io.Writer;
+import java.util.List;
 import java.util.concurrent.Callable;
 
 /**
@@ -17,14 +18,14 @@ public class NotIterableCode extends IterableCode {
   }
 
   @Override
-  public Writer execute(Writer writer, final Object[] scopes) {
+  public Writer execute(Writer writer, final List<Object> scopes) {
     Object resolved = get(scopes);
     writer = handle(writer, resolved, scopes);
     appendText(writer);
     return writer;
   }
 
-  protected Writer handle(Writer writer, Object resolved, Object[] scopes) {
+  protected Writer handle(Writer writer, Object resolved, List<Object> scopes) {
     if (resolved instanceof Callable) {
       writer = handleCallable(writer, (Callable) resolved, scopes);
     } else {
@@ -34,12 +35,12 @@ public class NotIterableCode extends IterableCode {
   }
 
   @Override
-  protected Writer execute(Writer writer, Object resolve, Object[] scopes) {
+  protected Writer execute(Writer writer, Object resolve, List<Object> scopes) {
     return oh.falsey(this, writer, resolve, scopes);
   }
 
   @Override
-  public Writer next(Writer writer, Object object, Object[] scopes) {
+  public Writer next(Writer writer, Object object, List<Object> scopes) {
     return run(writer, scopes);
   }
 }
