@@ -1,8 +1,10 @@
 package com.github.mustachejava.reflect;
 
+import java.util.List;
+
 /**
  * Simple specialization of Predicate
  */
 public interface Guard {
-  boolean apply(Object[] input);
+  boolean apply(List<Object> input);
 }
