@@ -37,13 +37,9 @@ public class NodeValue {
   public boolean equals(Object o) {
     if (this == o) return true;
     if (o == null || getClass() != o.getClass()) return false;
-
     NodeValue nodeValue = (NodeValue) o;
-
-    if (list != null ? !list.equals(nodeValue.list) : nodeValue.list != null) return false;
-    if (value != null ? !value.equals(nodeValue.value) : nodeValue.value != null) return false;
-
-    return true;
+    return !(list != null ? !list.equals(nodeValue.list) : nodeValue.list != null) && 
+            !(value != null ? !value.equals(nodeValue.value) : nodeValue.value != null);
   }
 
   @Override
