@@ -1,8 +1,10 @@
 package com.github.mustachejava.util;
 
+import java.util.List;
+
 /**
  * Call a wrapped name on a set of scopes.
  */
 public interface Wrapper {
-  Object call(Object[] scopes) throws GuardException;
+  Object call(List<Object> scopes) throws GuardException;
 }
