@@ -2,6 +2,8 @@ package com.github.mustachejava;
 
 import com.github.mustachejava.codes.IterableCode;
 import com.github.mustachejava.codes.PartialCode;
+import com.github.mustachejava.codes.ValueCode;
+import com.github.mustachejava.codes.WriteCode;
 import com.github.mustachejava.functions.CommentFunction;
 import com.github.mustachejava.reflect.ReflectionObjectHandler;
 import com.github.mustachejava.reflect.SimpleObjectHandler;
@@ -9,8 +11,6 @@ import com.github.mustachejava.resolver.DefaultResolver;
 import com.github.mustachejava.util.CapturingMustacheVisitor;
 import com.github.mustachejavabenchmarks.JsonCapturer;
 import com.github.mustachejavabenchmarks.JsonInterpreterTest;
-import com.google.common.base.Function;
-import com.google.common.collect.ImmutableMap;
 import junit.framework.TestCase;
 import org.codehaus.jackson.JsonGenerator;
 import org.codehaus.jackson.JsonNode;
@@ -18,22 +18,14 @@ import org.codehaus.jackson.map.MappingJsonFactory;
 import org.junit.Assert;
 import org.junit.Test;
 
-import java.io.BufferedReader;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.IOException;
-import java.io.InputStreamReader;
-import java.io.OutputStreamWriter;
-import java.io.Reader;
-import java.io.StringReader;
-import java.io.StringWriter;
-import java.io.Writer;
+import java.io.*;
 import java.util.*;
 import java.util.concurrent.Callable;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Executors;
 import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.function.Function;
 
 /**
  * Tests for the compiler.
@@ -275,7 +267,7 @@ public class InterpreterTest extends TestCase {
       mustacheFactory.setObjectHandler(new SimpleObjectHandler());
     }
     final Mustache defaultMustache = mustacheFactory.compile(new StringReader("{{#okGenerator.isItOk}}{{okGenerator.isItOk}}{{/okGenerator.isItOk}}"), "Test template");
-    final Map<String, Object> params = new HashMap<String, Object>();
+    final Map<String, Object> params = new HashMap<>();
     params.put("okGenerator", new OkGenerator());
     final Writer writer = new StringWriter();
     defaultMustache.execute(writer, params);
@@ -337,30 +329,21 @@ public class InterpreterTest extends TestCase {
 
     m.execute(sw, new Object() {
       Iterable list = Arrays.asList(
-              new Callable<Object>() {
-                @Override
-                public Object call() throws Exception {
-                  cdl1.await();
-                  sb.append("How");
-                  return "How";
-                }
+              () -> {
+                cdl1.await();
+                sb.append("How");
+                return "How";
               },
-              new Callable<Object>() {
-                @Override
-                public Object call() throws Exception {
-                  cdl2.await();
-                  sb.append("are");
-                  cdl1.countDown();
-                  return "are";
-                }
+              (Callable<Object>) () -> {
+                cdl2.await();
+                sb.append("are");
+                cdl1.countDown();
+                return "are";
               },
-              new Callable<Object>() {
-                @Override
-                public Object call() throws Exception {
-                  sb.append("you?");
-                  cdl2.countDown();
-                  return "you?";
-                }
+              () -> {
+                sb.append("you?");
+                cdl2.countDown();
+                return "you?";
               }
       );
     }).close();
@@ -375,36 +358,22 @@ public class InterpreterTest extends TestCase {
     StringWriter sw = new StringWriter();
     long start = System.currentTimeMillis();
     Writer execute = m.execute(sw, new Object() {
-      Callable<Object> a = new Callable<Object>() {
-        @Override
-        public Object call() throws Exception {
-          Thread.sleep(300);
-          return "How";
-        }
+      Callable<Object> a = () -> {
+        Thread.sleep(300);
+        return "How";
       };
-      Callable<Object> b = new Callable<Object>() {
-        @Override
-        public Object call() throws Exception {
-          Thread.sleep(200);
-          return "are";
-        }
+      Callable<Object> b = () -> {
+        Thread.sleep(200);
+        return "are";
       };
-      Callable<Object> c = new Callable<Object>() {
-        @Override
-        public Object call() throws Exception {
-          Thread.sleep(100);
-          return "you?";
-        }
+      Callable<Object> c = () -> {
+        Thread.sleep(100);
+        return "you?";
       };
-      Callable<Function> caps = new Callable<Function>() {
+      Callable<Function> caps = () -> new Function() {
         @Override
-        public Function call() throws Exception {
-          return new Function() {
-            @Override
-            public Object apply(Object o) {
-              return o.toString().toUpperCase();
-            }
-          };
+        public Object apply(Object o) {
+          return o.toString().toUpperCase();
         }
       };
     });
@@ -419,26 +388,17 @@ public class InterpreterTest extends TestCase {
     Mustache m = c.compile("latchedtest.html");
     StringWriter sw = new StringWriter();
     Writer execute = m.execute(sw, new Object() {
-      Callable<Object> nest = new Callable<Object>() {
-        @Override
-        public Object call() throws Exception {
-          Thread.sleep(300);
-          return "How";
-        }
+      Callable<Object> nest = () -> {
+        Thread.sleep(300);
+        return "How";
       };
-      Callable<Object> nested = new Callable<Object>() {
-        @Override
-        public Object call() throws Exception {
-          Thread.sleep(200);
-          return "are";
-        }
+      Callable<Object> nested = () -> {
+        Thread.sleep(200);
+        return "are";
       };
-      Callable<Object> nestest = new Callable<Object>() {
-        @Override
-        public Object call() throws Exception {
-          Thread.sleep(100);
-          return "you?";
-        }
+      Callable<Object> nestest = () -> {
+        Thread.sleep(100);
+        return "you?";
       };
     });
     execute.close();
@@ -477,11 +437,41 @@ public class InterpreterTest extends TestCase {
     assertEquals(getContents(root, "isempty.txt"), sw.toString());
   }
 
+  public void testOptional() throws IOException {
+    MustacheFactory c = createMustacheFactory();
+    Mustache m = c.compile(new StringReader("{{person}}{{#person}} is present{{/person}}{{^person}}Is not present{{/person}}"), "test");
+    StringWriter sw = new StringWriter();
+    m.execute(sw, new Object() {
+      Optional<String> person = Optional.of("Test");
+    });
+    assertEquals("Test is present", sw.toString());
+    sw = new StringWriter();
+    m.execute(sw, new Object() {
+      Optional<String> person = Optional.empty();
+    });
+    assertEquals("Is not present", sw.toString());
+  }
+
+  public void testComment() throws IOException {
+    MustacheFactory c = createMustacheFactory();
+    Mustache m = c.compile(new StringReader("{{#process}}{{!comment}}{{/process}}"), "");
+    StringWriter sw = new StringWriter();
+    m.execute(sw, new Object() {
+      TemplateFunction process = new TemplateFunction() {
+        @Override
+        public String apply(String s) {
+          return s.replace("{", "[");
+        }
+      };
+    });
+    assertEquals("[[!comment}}", sw.toString());
+  }
+
   public void testNumber0IsFalse() throws IOException {
     DefaultMustacheFactory c = createMustacheFactory();
     c.setObjectHandler(new ReflectionObjectHandler() {
       @Override
-      public Writer falsey(Iteration iteration, Writer writer, Object object, Object[] scopes) {
+      public Writer falsey(Iteration iteration, Writer writer, Object object, List<Object> scopes) {
         if (object instanceof Number) {
           if (((Number) object).intValue() == 0) {
             return iteration.next(writer, object, scopes);
@@ -491,7 +481,7 @@ public class InterpreterTest extends TestCase {
       }
 
       @Override
-      public Writer iterate(Iteration iteration, Writer writer, Object object, Object[] scopes) {
+      public Writer iterate(Iteration iteration, Writer writer, Object object, List<Object> scopes) {
         if (object instanceof Number) {
           if (((Number) object).intValue() == 0) {
             return writer;
@@ -581,12 +571,7 @@ public class InterpreterTest extends TestCase {
     StringWriter sw = new StringWriter();
     m.execute(sw, new Object() {
       public TemplateFunction i() {
-        return new TemplateFunction() {
-          @Override
-          public String apply(String s) {
-            return s;
-          }
-        };
+        return s -> s;
       }
     });
     assertEquals("This is not interesting.", sw.toString());
@@ -754,12 +739,9 @@ public class InterpreterTest extends TestCase {
       String description;
 
       Callable<String> desc() throws InterruptedException {
-        return new Callable<String>() {
-          @Override
-          public String call() throws Exception {
-            Thread.sleep(1000);
-            return description;
-          }
+        return () -> {
+          Thread.sleep(1000);
+          return description;
         };
       }
     }
@@ -800,8 +782,8 @@ public class InterpreterTest extends TestCase {
     String template = "Value: {{value}}";
     Mustache mustache = new DefaultMustacheFactory().compile(new StringReader(
             template), "test");
-    Map<String, String> emptyMap = new HashMap<String, String>();
-    Map<String, String> map = new HashMap<String, String>();
+    Map<String, String> emptyMap = new HashMap<>();
+    Map<String, String> map = new HashMap<>();
     map.put("value", "something");
 
     // map doesn't have an entry for 'value', lookup will fail
@@ -818,7 +800,7 @@ public class InterpreterTest extends TestCase {
     MustacheFactory mf = createMustacheFactory();
     Mustache compile = mf.compile("relative/paths.html");
     StringWriter sw = new StringWriter();
-    compile.execute(sw, null).close();
+    compile.execute(sw, "").close();
     assertEquals(getContents(root, "relative/paths.txt"), sw.toString());
   }
 
@@ -826,7 +808,7 @@ public class InterpreterTest extends TestCase {
     MustacheFactory mf = createMustacheFactory();
     Mustache compile = mf.compile("relative/rootpath.html");
     StringWriter sw = new StringWriter();
-    compile.execute(sw, null).close();
+    compile.execute(sw, "").close();
     assertEquals(getContents(root, "relative/paths.txt"), sw.toString());
   }
 
@@ -834,7 +816,7 @@ public class InterpreterTest extends TestCase {
     MustacheFactory mf = createMustacheFactory();
     Mustache compile = mf.compile("relative/extension.html");
     StringWriter sw = new StringWriter();
-    compile.execute(sw, null).close();
+    compile.execute(sw, "").close();
     assertEquals(getContents(root, "relative/paths.txt"), sw.toString());
   }
 
@@ -937,44 +919,55 @@ public class InterpreterTest extends TestCase {
       @Override
       public MustacheVisitor createMustacheVisitor() {
         DefaultMustacheVisitor visitor = new DefaultMustacheVisitor(this);
-        visitor.addPragmaHandler("pragma", new PragmaHandler() {
-          @Override
-          public Code handle(TemplateContext tc, String pragma, String args) {
-            if (pragma.equals("pragma") && args.equals("1 2 3")) {
-              found.set(true);
-            }
-            return null;
+        visitor.addPragmaHandler("pragma", (tc, pragma, args) -> {
+          if (pragma.equals("pragma") && args.equals("1 2 3")) {
+            found.set(true);
           }
+          return null;
         });
         return visitor;
       }
     };
     Mustache m = mf.compile(new StringReader("Pragma: {{% pragma 1 2 3 }}"), "testPragma");
     StringWriter sw = new StringWriter();
-    m.execute(sw, null).close();
+    m.execute(sw, "").close();
     // Values ignored as if it didn't exist at all
     assertEquals("Pragma: ", sw.toString());
     assertTrue(found.get());
   }
 
+  public void testPragmaWhiteSpaceHandling() throws IOException {
+    DefaultMustacheFactory mf = new DefaultMustacheFactory() {
+      @Override
+	  public MustacheVisitor createMustacheVisitor() {
+        DefaultMustacheVisitor visitor = new DefaultMustacheVisitor(this);
+	    // Add a pragma handler that simply renders a dash to visualize the output
+		visitor.addPragmaHandler("pragma", (tc, pragma, args) ->  new WriteCode(tc, this, "-"));
+        return visitor;
+      }
+	};
+
+	Mustache m = mf.compile(new StringReader(" {{% pragma}} {{% pragma}} "), "testPragma");
+	StringWriter sw = new StringWriter();
+	m.execute(sw, "").close();
+
+	// Pragma rendering should preserve template whitespace
+	assertEquals(" - - ", sw.toString());
+  }
+
   public void testNotIterableCallable() throws IOException {
     MustacheFactory mf = createMustacheFactory();
     Mustache m = mf.compile(new StringReader("{{^value}}test{{/value}}"), "testNotIterableCallable");
     StringWriter sw = new StringWriter();
     m.execute(sw, new Object() {
-      Callable value = new Callable() {
-        @Override
-        public Object call() throws Exception {
-          return null;
-        }
-      };
+      Callable value = () -> null;
     }).close();
     // Values ignored as if it didn't exist at all
     assertEquals("test", sw.toString());
   }
 
   public static class AccessTrackingMap extends HashMap<String, String> {
-    Set<Object> accessed = new HashSet<Object>();
+    Set<Object> accessed = new HashSet<>();
 
     @Override
     public String get(Object key) {
@@ -983,7 +976,7 @@ public class InterpreterTest extends TestCase {
     }
 
     public void check() {
-      Set<String> keyset = new HashSet<String>(keySet());
+      Set<String> keyset = new HashSet<>(keySet());
       keyset.removeAll(accessed);
       if (!keyset.isEmpty()) {
         throw new MustacheException("All keys in the map were not accessed");
@@ -1064,12 +1057,7 @@ public class InterpreterTest extends TestCase {
         }
       };
       String test2 = "test";
-      Function f = new Function() {
-        @Override
-        public Object apply(Object o) {
-          return null;
-        }
-      };
+      Function f = o -> null;
       CommentFunction comment = new CommentFunction();
     }).close();
     // Values ignored as if it didn't exist at all
@@ -1094,7 +1082,7 @@ public class InterpreterTest extends TestCase {
     MustacheFactory mf = createMustacheFactory();
     Mustache compile = mf.compile("relative/dotdot.html");
     StringWriter sw = new StringWriter();
-    compile.execute(sw, null).close();
+    compile.execute(sw, "").close();
     assertEquals(getContents(root, "uninterestingpartial.html"), sw.toString());
   }
 
@@ -1107,7 +1095,7 @@ public class InterpreterTest extends TestCase {
     };
     Mustache compile = mf.compile("relative/nonrelative.html");
     StringWriter sw = new StringWriter();
-    compile.execute(sw, null).close();
+    compile.execute(sw, "").close();
     assertEquals(getContents(root, "nonrelative.html"), sw.toString());
   }
 
@@ -1130,7 +1118,7 @@ public class InterpreterTest extends TestCase {
       }
     };
     StringWriter sw = new StringWriter();
-    mf.compile("overrideextension.html").execute(sw, null).close();
+    mf.compile("overrideextension.html").execute(sw, "").close();
     assertEquals("not interesting.", sw.toString());
   }
 
@@ -1156,7 +1144,7 @@ public class InterpreterTest extends TestCase {
   public void testImplicitIteratorNoScope() throws IOException {
     Mustache test = new DefaultMustacheFactory().compile(new StringReader("{{.}}"), "test");
     StringWriter sw = new StringWriter();
-    test.execute(sw, null).close();
+    test.execute(sw, "").close();
     assertEquals("", sw.toString());
     StringWriter sw2 = new StringWriter();
     test.execute(sw2, new Object[0]).close();
@@ -1223,12 +1211,7 @@ public class InterpreterTest extends TestCase {
     StringWriter sw = new StringWriter();
     mustache.execute(sw, new Object() {
       public TemplateFunction parse() {
-        return new TemplateFunction() {
-          @Override
-          public String apply(String s){
-            return "blablabla {{anotherVar}}, blablabla {{yetAnotherVar}}";
-          }
-        };
+        return s -> "blablabla {{anotherVar}}, blablabla {{yetAnotherVar}}";
       }
       String anotherVar = "banana";
       String yetAnotherVar = "apple";
@@ -1246,8 +1229,13 @@ public class InterpreterTest extends TestCase {
           public void iterable(final TemplateContext templateContext, String variable, Mustache mustache) {
             list.add(new IterableCode(templateContext, df, mustache, variable) {
               Binding binding = oh.createBinding("params", templateContext, this);
-              protected Writer handleFunction(Writer writer, Function function, Object[] scopes) {
-                return super.handleFunction(writer, function, addScope(scopes, binding.get(scopes)));
+              protected Writer handleFunction(Writer writer, Function function, List<Object> scopes) {
+                boolean added = addScope(scopes, binding.get(scopes));
+                try {
+                  return super.handleFunction(writer, function, scopes);
+                } finally {
+                  if (added) scopes.remove(scopes.size() - 1);
+                }
               }
             });
           }
@@ -1258,17 +1246,13 @@ public class InterpreterTest extends TestCase {
     StringWriter sw = new StringWriter();
     mustache.execute(sw, new Object() {
       public TemplateFunction parse() {
-        return new TemplateFunction() {
-          @Override
-          public String apply(String s){
-            return "blablabla {{anotherVar}}, blablabla {{yetAnotherVar}}";
-          }
-        };
+        return s -> "blablabla {{anotherVar}}, blablabla {{yetAnotherVar}}";
+      }
+      Map<String, Object> params = new HashMap<>();
+      {
+        params.put("anotherVar", "banana");
+        params.put("yetAnotherVar", "apple");
       }
-      Map<String, Object> params = ImmutableMap.<String, Object>builder()
-          .put("anotherVar", "banana")
-          .put("yetAnotherVar", "apple")
-          .build();
     });
     assertEquals("blablabla banana, blablabla apple", sw.toString());
   }
@@ -1277,12 +1261,45 @@ public class InterpreterTest extends TestCase {
     DefaultMustacheFactory mustacheFactory = new DefaultMustacheFactory();
     Reader reader = new StringReader("value=${some.value}");
     Mustache mustache = mustacheFactory.compile(reader, "maven", "${", "}");
-    Map<String, String> properties = new HashMap<String, String>();
+    Map<String, String> properties = new HashMap<>();
     properties.put("some.value", "some.value");
     StringWriter writer = new StringWriter();
     mustache.execute(writer, new Object[]{properties}).close();
     Assert.assertEquals("value=some.value", writer.toString());
   }
+  
+  public void testLeavingAloneMissingVariables() throws IOException {
+    DefaultMustacheFactory dmf = new DefaultMustacheFactory(root) {
+      @Override
+      public MustacheVisitor createMustacheVisitor() {
+        return new DefaultMustacheVisitor(this) {
+          @Override
+          public void value(TemplateContext tc, String variable, boolean encoded) {
+            list.add(new ValueCode(tc, df, variable, encoded) {
+              @Override
+              public Writer execute(Writer writer, List<Object> scopes) {
+                try {
+                  final Object object = get(scopes);
+                  if (object == null) {
+                    identity(writer);
+                  }
+                  return super.execute(writer, scopes);
+                } catch (Exception e) {
+                  throw new MustacheException("Failed to get value for " + name, e, tc);
+                }
+              }
+            });
+          }
+        };
+      }
+    };
+    Mustache test = dmf.compile(new StringReader("{{name}} - {{email}}"), "test");
+    StringWriter sw = new StringWriter();
+    Map map = new HashMap<>();
+    map.put("name", "Sam Pullara");
+    test.execute(sw, map).close();
+    assertEquals("Sam Pullara - {{email}}", sw.toString());
+  }
 
   private MustacheFactory init() {
     return createMustacheFactory();
@@ -1314,12 +1331,12 @@ public class InterpreterTest extends TestCase {
 
   @Test
   public void testMap() throws IOException {
-    ArrayList<Map<String, String>> fn = new ArrayList<Map<String, String>>();
-    Map<String, String> map1 = new HashMap<String, String>();
+    ArrayList<Map<String, String>> fn = new ArrayList<>();
+    Map<String, String> map1 = new HashMap<>();
     map1.put("name", "firstName");
     map1.put("last", "lastName");
     fn.add(map1);
-    Map<String, String> map2 = new HashMap<String, String>();
+    Map<String, String> map2 = new HashMap<>();
     map2.put("name", "firstName 1");
     map2.put("last", "lastName 1");
     fn.add(map2);
