@@ -13,15 +13,10 @@ import java.util.concurrent.Callable;
 */
 public class ParallelComplexObject {
   String header = "Colors";
-  Callable<List<Color>> item = new Callable<List<Color>>() {
-    @Override
-    public List<Color> call() throws Exception {
-      return Arrays.asList(
-              new Color("red", true, "#Red"),
-              new Color("green", false, "#Green"),
-              new Color("blue", false, "#Blue"));
-    }
-  };
+  Callable<List<Color>> item = () -> Arrays.asList(
+          new Color("red", true, "#Red"),
+          new Color("green", false, "#Green"),
+          new Color("blue", false, "#Blue"));
 
   boolean list() {
     return true;
